/*
 * IndexSetContainer.hpp
 *
 *  Created on: Jul 3, 2012
 *      Author: heber
 */

#ifndef INDEXSETCONTAINER_HPP_
#define INDEXSETCONTAINER_HPP_


// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <boost/shared_ptr.hpp>
#include <vector>

#include "IndexSet.hpp"
#include "SortedVector.hpp"

class IndexSetContainerTest;
class KeySetsContainer;

/**
  * As the IndexSet is supposed to appear in many maps and Containers as it may
  * be contained in other IndexSet's, we store it always as a shared_ptr. This
  * Container is then responsible that the ordering within the container still
  * allows for fast access due to sorted appearance not by order in memory but
  * order defined by IndexSet comparison operators.
  *
  */

//!> typedef for IndexSet instances stored in SortedVector
class IndexSetContainer : public SortedVector<IndexSet> {
  //!> grant unit test access to protected members of SortedVector
  friend class IndexSetContainerTest;
public:
  //!> typedef for IndexSetContainer wrapped in shared_ptr
  typedef boost::shared_ptr<IndexSetContainer> ptr;

  /** Constructor from an unsorted vector of instances.
   *
   * @param _container unsorted vector of instances
   */
  IndexSetContainer(const SortedVector<IndexSet>::Container_t &_container) :
    SortedVector<IndexSet>(_container)
  {}

  /** Constructor from an unsorted vector of instances.
   *
   * @param _container unsorted vector of instances
   */
  IndexSetContainer(const std::vector<IndexSet> &_container) :
    SortedVector<IndexSet>(_container)
  {}

  /** Constructor from a single instance.
   *
   * @param _instance single instance
   */
  IndexSetContainer(IndexSet::ptr &_instance) :
    SortedVector<IndexSet>(_instance)
  {}

  /** Conversion constructor from a KeySetConntainer.
   *
   * We create the super set of all found key sets and then each single set,
   * while always dropping indices equal to -1.
   *
   * @param _keysets keysets to turn into IndexSets
   */
  explicit IndexSetContainer(const KeySetsContainer &_keysets);

private:
  /** Helper function to get the super set out of a KeySetsContainer.
   *
   * @param _keysets keysets to construct super set for
   * @return IndexSet that contains each index once that appears in the sets of \a _keysets
   */
  IndexSet::ptr& createSuperSet(const KeySetsContainer &_keysets) const;

  //!> static instance for AllIndices such that createSuperSet may return a reference.
  static IndexSet::ptr AllIndices;
};

//!> typedef for IndexSet instance stored as shared_ptr in SortedVector
typedef IndexSetContainer::T_ptr IndexSet_ptr;


#endif /* INDEXSETCONTAINER_HPP_ */
